// Programm zur Erstellung von verlinkten html-Seiten für einen Blog
// Dominik Zobel, Juli 2013

// Standardeingabe
#include <iostream>
#include <fstream>
#include <string>
#include <cstring>

// Listen unbekannter Länge und Vektoren
#include <list>
#include <vector>

// Allgemeines Sortieren
#include <algorithm>

// Alles in Kleinbuchstaben (tolower)
#include <cctype>

// String -> Integer zum Vergleichen von Daten
#include <sstream>

// Zum Ordner durchsuchen und Inhalt auslesen
#include <dirent.h>
#include <stdlib.h>

// Existenz von Ordnern überprüfen
#include <sys/stat.h>

// Exceptions abfangen
#include <stdexcept>


using namespace std;


// --- Einstellungen, die angepasst werden können
// Ersetzungen vom Präprozessor (idealerweise auszulagern in eine Konfigurationsdatei)
#define _praep_maxEintraegeIndex_ 5                        // Anzahl der Einträge, die auf den Index-Seiten angezeigt werden sollen

#define _praep_schlagwortnr_ 1                             // Soll die Anzahl der Schlagworte in Klammern angezeigt werden (1) oder nicht (0)

// Globale Variablen (in eigenem Namensbereich)
namespace globalvar {
   // Aus diesem Unterordner zum Programm werden die Quelldateien im "beispiel.txt" eingelesen
   const std::string ordner_einzulesen =    "quelldateien";

   // In diesem Unterordner wird die Ausgabe erzeugt
   const std::string ordner_ausgabe =       "www";

   // Im Unterordner der Ausgabe werden folgende Ordner für die Eintragsdateien bzw. Schlagwortdateien erstellt
   const std::string ordner_eintraege =     "eintraege";
   const std::string ordner_schlagwoerter = "schlagwoerter";
}
// ----------------------------------------------------


// Auflistung aller Funktionen - Kommentierte benötigen aktuell keine frühzeitige Initialisierung
bool KriteriumNachAlphabetSortieren( string eins, string zwei );
bool KriteriumNachDatumSortieren( string const &zuPruefendesDatum, string const &referenzDatum );
int StringToInteger( string umzuwandeln );
string IntegerToString( int umzuwandeln );
int TagInhaltExtrahieren( string zuOeffnendeDatei, string gesuchterTag, string &taginhalt );
int ZeileUntersuchenUndAusgeben( string &ausgangs_zeile, string &vorMuster, string &dasMuster, string &nachMuster, string muster_beginn, string muster_ende );
// void zeige_benutzungshinweise();
// int ueberpruefe_existenz_von_ordner( string zu_ueberpruefender_ordner );
// int BedingungenUeberpruefen();
// int DateiOeffnen(, DateiHandleStruktur &meineDateiHandleStruktur, string dateiname string zugriff );
// int LeseDateienAusVerzeichnis( list<string> &dateiliste );
// void InternesMusterErsetzen( string &muster, vector<string> &pfadVon, string &rueckgabe );
// int ErstelleZieldateien( Gesamtstruktur &meineStruktur );
// int main( int argc, char* argv[] );


// --- Definition der Klassen
class Eintrag {
   private:
      string quelldateiname_;
      string kerndateiname_;
      string titel_;
      string datum_;
      string schlagwoerter_;
   public:
      Eintrag() {
         quelldateiname_ = "quelldateiname nicht gesetzt";
         kerndateiname_  = "zieldateiname nicht gesetzt";
         titel_          = "titel nicht gesetzt";
         datum_          = "datum nicht gesetzt";
         schlagwoerter_  = "schlagwoerter nicht gesetzt";
      }
      ~Eintrag(){}
      int LeseTaginhaltAusDatei( string dateiname ) {
         cout << "Lese aus Datei \"" + dateiname + "\"\n";
         if ( TagInhaltExtrahieren( globalvar::ordner_einzulesen + "/" + quelldateiname_, "titel", titel_ ) )
            return 1;

         if ( TagInhaltExtrahieren( globalvar::ordner_einzulesen + "/" + quelldateiname_, "datum", datum_ ) )
            return 1;

         if ( TagInhaltExtrahieren( globalvar::ordner_einzulesen + "/" + quelldateiname_, "schlagwoerter", schlagwoerter_ ) )
            return 1;                                      // Die Quelldatei muss im Format .txt vorliegen (drei Zeichen),
                                                           // so dass folgender Befehl die Dateiendung einfach in html ändert
         kerndateiname_ = quelldateiname_.substr( 0, quelldateiname_.size()-4 );

         return 0;
      }
      int SetzeDateiname( string dateiname ) {
         quelldateiname_ = dateiname;
         return 0;
      }
      const string& HoleQuellname() {
         return quelldateiname_;
      }
      const string& HoleKernname() {
         return kerndateiname_;
      }
      const string& HoleTitel() {
         return titel_;
      }
      const string& HoleDatum() {
         return datum_;
      }
      const string& HoleSchlagwoerter() {
         return schlagwoerter_;
      }
};


class Gesamtstruktur {
   private:
      vector<Eintrag> Eintragslistenvektor;
      vector<Eintrag> Archivlistenvektor;
      vector<string> schlagwortliste;
      vector<int> schlagwort_anzahl;
      string liste_html_uebersicht_;
   public:
      Gesamtstruktur() {
         liste_html_uebersicht_ = "";
      }
      ~Gesamtstruktur(){}
      int EintragHinzufuegen( string nameEintrag ) {
         Eintrag neuerEintrag;
         neuerEintrag.SetzeDateiname( nameEintrag );

         if ( neuerEintrag.LeseTaginhaltAusDatei( nameEintrag ) ) {
            // Sollte bei automatisch eingelesenen Dateinamen mit Leserechten niemals vorkommen
            cout << "Warnung: Konnte die Daten aus \"" << nameEintrag << "\" nicht lesen\n";
            return 1;
         }

         Eintragslistenvektor.push_back( neuerEintrag );
         return 0;
      }
      const vector<Eintrag> EintraegeAusgebenAlphabetisch() {
         return Eintragslistenvektor;
      }
      const vector<Eintrag> EintraegeAusgebenDatum() {
         return Archivlistenvektor;
      }
      const vector<string> AusgabeSchlagworte() {
         return schlagwortliste;
      }
      const vector<int> AusgabeAnzahlSchlagworte() {
         return schlagwort_anzahl;
      }
      string HTMLauflistung( string typ, vector<string> pfadZu, string ausgangsdateiname ) {
         if ( typ == "")
            return HTMLneues( pfadZu, ausgangsdateiname );

         if ( typ == "-al")
            return HTMLalphabetisch( pfadZu, ausgangsdateiname );

         if ( typ == "-ar")
            return HTMLarchiv( pfadZu, ausgangsdateiname );

         if ( typ == "-sc")
            return HTMLschlagworte( pfadZu, ausgangsdateiname );

         cout << "Warnung: Typ nicht erkannt, gebe Zeichenkette \"nicht erkannt\" zurück\n";
         return "nicht erkannt";
      }
      string& HTMLneues( vector<string> pfadZu, string ausgangsdateiname ) {
         liste_html_uebersicht_ = "<a href=\"" + pfadZu[0] + "index.html\">Neuste Einträge</a><br />\n";
         liste_html_uebersicht_.append( "         <a href=\"" + ausgangsdateiname + "-al.html\">Alphabetisch</a><br />\n" +
                                        "         <a href=\"" + ausgangsdateiname + "-ar.html\">Archiv</a><br />\n" +
                                        "         <a href=\"" + ausgangsdateiname + "-sc.html\">Schlagwörter</a>" );

         return liste_html_uebersicht_;
      }
      string& HTMLalphabetisch( vector<string> pfadZu, string ausgangsdateiname ) {
         liste_html_uebersicht_ = "<a href=\"" + pfadZu[0] + "index.html\">Neuste Einträge</a><br />\n";
         liste_html_uebersicht_.append( "         <a href=\"" + ausgangsdateiname + ".html\">Alphabetisch</a><br />\n         <ul class=\"uebersichtsliste\">\n" );
         for ( int i = 0; i < Eintragslistenvektor.size(); i++ ) {
            liste_html_uebersicht_.append( "            <li><a href=\"" + pfadZu[2] + Eintragslistenvektor[i].HoleKernname() + "-al.html\">" +
                                           Eintragslistenvektor[i].HoleTitel() + "</a></li>\n" );
         }
         liste_html_uebersicht_.append( "         </ul>\n         <a href=\"" + ausgangsdateiname +
                                        "-ar.html\">Archiv</a><br />\n         <a href=\"" + ausgangsdateiname +
                                        "-sc.html\">Schlagwörter</a>" );

         return liste_html_uebersicht_;
      }
      string& HTMLarchiv( vector<string> pfadZu, string ausgangsdateiname ) {
         string monatsnamen[] = { "Januar", "Februar", "März", "April", "Mai", "Juni", "Juli", "August", "September", "Oktober", "November", "Dezember" };
         int letzter_monat = 0;
         string letztes_jahr  = "";
         liste_html_uebersicht_ = "<a href=\"" + pfadZu[0] + "index.html\">Neuste Einträge</a><br />\n";
         liste_html_uebersicht_.append( "         <a href=\"" + ausgangsdateiname + "-al.html\">Alphabetisch</a><br />\n         <a href=\"" +
                                        ausgangsdateiname + ".html\">Archiv</a><br />\n         <ul class=\"archiv\">\n" );

         for ( int i = 0 ; i < Archivlistenvektor.size() ; i++) {
            string aktuelles_jahr = Archivlistenvektor[i].HoleDatum().substr( 6 );
            if ( aktuelles_jahr != letztes_jahr ) {        // Immer wenn das Jahr wechselt, soll das aktuelle Jahr ausgegeben werden
               if ( letztes_jahr != "" ) {
                  liste_html_uebersicht_.append( "               </ul>\n            </li>\n" );
               }
               liste_html_uebersicht_.append( "            <li class=\"archivjahr\">" + aktuelles_jahr + "</li>\n" );
               letztes_jahr = aktuelles_jahr;
               letzter_monat = 0;                          // Wenn das Jahr wechselt, wechselt auch der Monat
            }
            int monats_zaehler = StringToInteger( Archivlistenvektor[i].HoleDatum().substr( 3,2 ) );
            if ( letzter_monat != monats_zaehler ) {       // Immer wenn der Monat wechselt, soll das Format angepasst ...
               if ( letzter_monat != 0 ) {
                  liste_html_uebersicht_.append( "               </ul>\n            </li>\n" );
               }                                           // ... und der Monat als neuer Punkt ausgeschrieben werden
               liste_html_uebersicht_.append( "            <li>" + monatsnamen[monats_zaehler-1] + "\n               <ul class=\"archivliste\">\n" );

               letzter_monat = monats_zaehler;
            }
                                                           // Der "eigentliche" Eintrag mit Tag, Titel und Link dorthin
            liste_html_uebersicht_.append( "                  <li>" + Archivlistenvektor[i].HoleDatum().substr( 0,2 ) + ".: <a href=\"" + pfadZu[2] +
                                           Archivlistenvektor[i].HoleKernname() + "-ar.html\">" + Archivlistenvektor[i].HoleTitel() + "</a></li>\n" );
         }

         liste_html_uebersicht_.append( "               </ul>\n            </li>\n         </ul>\n         <a href=\"" +
                                        ausgangsdateiname + "-sc.html\">Schlagwörter</a>" );

         return liste_html_uebersicht_;
      }
      string& HTMLschlagworte( vector<string> pfadZu, string ausgangsdateiname ) {
         int anzahl_in_klammern_anzeigen = _praep_schlagwortnr_;
         liste_html_uebersicht_ = "<a href=\"" + pfadZu[0] + "index.html\">Neuste Einträge</a><br />\n";
         liste_html_uebersicht_.append( "         <a href=\"" + ausgangsdateiname + "-al.html\">Alphabetisch</a><br />\n" +
                                        "         <a href=\"" + ausgangsdateiname + "-ar.html\">Archiv</a><br />\n" +
                                        "         <a href=\"" + ausgangsdateiname + ".html\">Schlagwörter</a>\n" +
                                        "         <ul class=\"uebersichtsliste\">\n" );
         string temp_klein;
         for ( int i = 0 ; i < schlagwortliste.size() ; i++) {
            liste_html_uebersicht_.append( "            <li><a href=\"" + pfadZu[1] + "schlagwort-" );
            temp_klein = schlagwortliste[i];
            transform( temp_klein.begin(), temp_klein.end(), temp_klein.begin(), ::tolower );
            liste_html_uebersicht_.append( temp_klein + "-sc.html\">" + schlagwortliste[i] + "</a>" );
            if ( anzahl_in_klammern_anzeigen )
               liste_html_uebersicht_.append( " (" + IntegerToString( schlagwort_anzahl[i] ) + ")" );

            liste_html_uebersicht_.append( "</li>\n" );
         }
         liste_html_uebersicht_.append( "         </ul>" );
         return liste_html_uebersicht_;
      }
      int ErstelleInterneSchlagwortliste() {
         string temp_schlagworte;
         vector<string> temp_schlagwortliste;
         for ( int i = 0; i < Eintragslistenvektor.size(); i++ ) {
            temp_schlagworte = Eintragslistenvektor[i].HoleSchlagwoerter();
            size_t stelle_komma = temp_schlagworte.find( ", " );

            if ( stelle_komma != string::npos ) {
               while ( stelle_komma != string::npos ) {
                  string temp_anfang = temp_schlagworte.substr( 0, stelle_komma );
                                                           // Starte zwei Stellen weiter wegen dem Trennzeichen ", "
                  temp_schlagworte = temp_schlagworte.substr( stelle_komma+2 );
                  temp_schlagwortliste.push_back( temp_anfang );

                  stelle_komma = temp_schlagworte.find( ", " );
               }
            }

            temp_schlagwortliste.push_back( temp_schlagworte );
         }
                                                           // Liste sortieren und Mehrfacheinträge entfernen
         for ( int zaehler_swl = 1; zaehler_swl < temp_schlagwortliste.size(); zaehler_swl++ ) {
            int temp_zaehler_swl = zaehler_swl;
            while ( KriteriumNachAlphabetSortieren( temp_schlagwortliste[temp_zaehler_swl-1], temp_schlagwortliste[temp_zaehler_swl] ) ) {
               temp_schlagworte = temp_schlagwortliste[temp_zaehler_swl];
               temp_schlagwortliste[temp_zaehler_swl] = temp_schlagwortliste[temp_zaehler_swl-1];
               temp_schlagwortliste[temp_zaehler_swl-1] = temp_schlagworte;
               temp_zaehler_swl--;
               if ( temp_zaehler_swl == 0 )
                  break;
            }
         }

         schlagwortliste.clear();                          // Untersuchen, ob ein Eintrag doppelt oder mehrfach vorkommt.
         schlagwort_anzahl.clear();                        // Die Häufigkeit wird erst mit Hilfe von zaehler_anzahl bestimmt und
         temp_schlagworte = "";                            // schließlich in schlagwort_anzahl gespeichert.
         int zaehler_anzahl = 1;                           // Zum Entfernen der doppelten Einträge würde auch folgender Befehl reichen
         // schlagwortliste.erase( unique( schlagwortliste.begin(), schlagwortliste.end() ), schlagwortliste.end() );
         for ( int i = 0; i< temp_schlagwortliste.size(); i++ ) {
            if ( temp_schlagworte != temp_schlagwortliste[i] ) {
               schlagwortliste.push_back( temp_schlagwortliste[i] );
               if ( schlagwortliste.size() > 1 )
                  schlagwort_anzahl.push_back( zaehler_anzahl );
               temp_schlagworte = temp_schlagwortliste[i];
               zaehler_anzahl = 1;
            }
            else
               zaehler_anzahl++;
         }
         schlagwort_anzahl.push_back( zaehler_anzahl );

         return 0;
      }
      int ErstelleInterneAlphabetischeliste() {
         Eintrag tempEintrag;
         for ( int zaehler_eintrag = 1; zaehler_eintrag < Eintragslistenvektor.size(); zaehler_eintrag++ ) {
            int temp_zaehler_eintrag = zaehler_eintrag;
            while ( KriteriumNachAlphabetSortieren( Eintragslistenvektor[temp_zaehler_eintrag-1].HoleTitel(), Eintragslistenvektor[temp_zaehler_eintrag].HoleTitel() ) ) {
               tempEintrag = Eintragslistenvektor[temp_zaehler_eintrag];
               Eintragslistenvektor[temp_zaehler_eintrag] = Eintragslistenvektor[temp_zaehler_eintrag-1];
               Eintragslistenvektor[temp_zaehler_eintrag-1] = tempEintrag;
               temp_zaehler_eintrag--;
               if ( temp_zaehler_eintrag == 0 )
                  break;
            }
         }
         return 0;
      }
      int ErstelleInterneDatumsliste() {
         Archivlistenvektor = Eintragslistenvektor;
         Eintrag tempEintrag;
         for ( int zaehler_eintrag = 1; zaehler_eintrag < Archivlistenvektor.size(); zaehler_eintrag++ ) {
            int temp_zaehler_eintrag = zaehler_eintrag;
            while ( KriteriumNachDatumSortieren( Archivlistenvektor[temp_zaehler_eintrag-1].HoleDatum(), Archivlistenvektor[temp_zaehler_eintrag].HoleDatum() ) ) {
               tempEintrag = Archivlistenvektor[temp_zaehler_eintrag];
               Archivlistenvektor[temp_zaehler_eintrag] = Archivlistenvektor[temp_zaehler_eintrag-1];
               Archivlistenvektor[temp_zaehler_eintrag-1] = tempEintrag;
               temp_zaehler_eintrag--;
               if ( temp_zaehler_eintrag == 0 )
                  break;
            }
         }
         return 0;
      }
};


class DateiHandleStruktur {
   private:
      FILE* AktiveDatei;
      vector<FILE*> HandleListe_;
      char buf[BUFSIZ];
   public:
      DateiHandleStruktur() {};
      ~DateiHandleStruktur(){
         for ( int i = 0; i < HandleListe_.size(); i++) {
            fclose( HandleListe_[i] );
         }
      }
      void HandleEintragen( string dateiname, string zugriff ) {
         AktiveDatei = fopen( dateiname.c_str(), zugriff.c_str() );
         if ( AktiveDatei == NULL )
            throw runtime_error( "Konnte Datei nicht zum Lesen öffnen" );

         HandleListe_.push_back( AktiveDatei );
      }
      const vector<FILE*>& HandleListeAusgeben() {
         return HandleListe_;
      }
      int ZeileEinlesen( FILE* OffeneDatei, string &zeile ) {
                                                           // Sofern die Zeile nicht die letzte ist
         if ( fgets( buf, sizeof(buf), OffeneDatei ) != NULL ) {
            buf[strlen(buf) - 1] = '\0';                   // Entferne das letzte Zeichen (Zeilenumbruch)
            zeile.assign( buf );
            // angelehnt an: http://stackoverflow.com/questions/8438686/convert-char-to-string-c
            return 0;
         }

         fseek( OffeneDatei, 0, SEEK_SET );                // Wieder an den Anfang zurücksetzen
         // Es ist wohl effizienter, Dateien nicht neu einzulesen, sondern sie in einer Variablen zu speichern. Das nächste Mal ...
         return 1;                                         // Konnte keine Zeile einlesen
      }
      string DateiEinlesen( FILE* EinzulesendeDatei, Gesamtstruktur &dieStruktur, string kernname, string dateisuffix, vector<string> pfadVon, bool istSchlagwortdatei ) {
         string ausgabe     = "";
         string tempzeile_  = "";
         string vorMuster_  = "";
         string dasMuster_  = "";
         string nachMuster_ = "";

         int zeilenuntersuchung = 1;

         while ( ! ZeileEinlesen( EinzulesendeDatei, tempzeile_ ) ) {
            while ( tempzeile_.size() != 0 ) {
               zeilenuntersuchung = ZeileUntersuchenUndAusgeben( tempzeile_, vorMuster_, dasMuster_, nachMuster_, "=__", "__=" );

               if ( zeilenuntersuchung == 1 )
                  cout << "Warnung: Probleme beim Untersuchen der Zeile - Fehlerhafter Dateiinhalt möglich\n";

               ausgabe.append( vorMuster_ );
               if ( dasMuster_ != "" ) {
                  int musterCookie = 0;
                  if ( dasMuster_ == "zum_hauptverzeichnis" ) {
                     ausgabe.append( pfadVon[0] );
                     musterCookie = 1;
                  }

                  if ( dasMuster_ == "uebersicht" ) {
                     if ( istSchlagwortdatei ) {
                        transform( kernname.begin(), kernname.end(), kernname.begin(), ::tolower );
                        kernname = "schlagwort-" + kernname;
                     }

                     string uebersicht = dieStruktur.HTMLauflistung( dateisuffix, pfadVon, kernname );
                     ausgabe.append( uebersicht );
                     musterCookie = 1;
                  }

                  if ( musterCookie == 0 )
                     cout << "Warnung: Muster \"" << dasMuster_ << "\" nicht implementiert" << endl;
               }

               tempzeile_ = nachMuster_;
            }

            ausgabe.append( "\n" );
         }

         return ausgabe;
      }
      int ZeichenAusgeben( FILE* OffeneDatei, string zeile ) {
         fputs( zeile.c_str(), OffeneDatei );
         return 0;
      }
};
// ----------------------------------------------------



// =================================================================
// ==========   S O R T I E R E N :     A L P H A B E T   ==========
// =================================================================
// ---
// Strings unabhängig von Groß-/Kleinschreibung sortieren (d.h. alle werden in Kleinbuchstaben umgewandelt)
bool KriteriumNachAlphabetSortieren( string eins, string zwei ) {
                                                           // ::tolower statt tolower nötig (aus globalen Namensraum)
   transform( eins.begin(), eins.end(), eins.begin(), ::tolower );
   transform( zwei.begin(), zwei.end(), zwei.begin(), ::tolower );
   return eins > zwei;
}



// =================================================================
// =============   S O R T I E R E N :     D A T U M   =============
// =================================================================
// ---
bool KriteriumNachDatumSortieren( string const &zuPruefendesDatum, string const &referenzDatum ) {
                                                           // Zuerst wird überprüft, ob beide Zeichenketten 10 Zeichen enthalten. Sie sollten im
                                                           // Format DD.MM.YYYY vorliegen, da sonst unerwartete Ergebnisse eintreten. Eine Rückgabe
   if ( zuPruefendesDatum.size() != 10 ) {                 // von 0 heißt, dass das zu prüfende Datum in der Zukunft des referenzDatums liegt
      cout << "Abbruch: Datum muss im Format DD.MM.YYYY sein. Aktuell " << zuPruefendesDatum << endl;
      return 2;
   }

   if ( referenzDatum.size() != 10 ) {
      cout << "Abbruch: Datum muss im Format DD.MM.YYYY sein. Aktuell " << referenzDatum << endl;
      return 2;
   }

   int eintrag_jahr = StringToInteger( zuPruefendesDatum.substr( 6 ) );
   int aktuell_jahr = StringToInteger( referenzDatum.substr( 6 ) );
   if ( eintrag_jahr < aktuell_jahr )                      // Zuerst werden die Jahre verglichen. Wenn eines eindeutig in einem vergangenem oder
      return 1;                                            // zukünftigem Jahr liegt, sind wir fertig. Bei Gleichheit werden die Monate überprüft
   else {                                                  // und analog werden nur bei Gleichheit auch die Tage überprüft.
      if ( eintrag_jahr > aktuell_jahr )
         return 0;

      int eintrag_monat = StringToInteger( zuPruefendesDatum.substr( 3,2 ) );
      int aktuell_monat = StringToInteger( referenzDatum.substr( 3,2 ) );
      if ( eintrag_monat < aktuell_monat )
         return 1;
      else {
         if ( eintrag_monat > aktuell_monat )
            return 0;

         int eintrag_tag = StringToInteger( zuPruefendesDatum.substr( 0,2 ) );
         int aktuell_tag = StringToInteger( referenzDatum.substr( 0,2 ) );
      if ( eintrag_tag < aktuell_tag )
         return 1;
      else
         return 0;
      }
   }
}



// =================================================================
// ============   S T R I N G    Z U    I N T E G E R   ============
// =================================================================
// ---
int StringToInteger( string umzuwandeln ) {
   stringstream ss( umzuwandeln );                         // Erzeuge einen Stream aus dem String
   int return_integer;
   ss >> return_integer;                                   // Weise dem Integer den Stream zu und gebe ihn zurück
   return return_integer;
}



// =================================================================
// ============   I N T E G E R    Z U    S T R I N G   ============
// =================================================================
// ---
string IntegerToString( int umzuwandeln ) {
   stringstream ss;
   ss << umzuwandeln;
   return ss.str();                                        // Erzeuge den (Rückgabe-)String mit der str() Funktion von stringstream
}



// =================================================================
// ========   T A G I N H A L T     E X T R A H I E R E N   ========
// =================================================================
// ---
int TagInhaltExtrahieren( string zuOeffnendeDatei, string gesuchterTag, string &taginhalt ) {
   string zeile = "";
   string muster_anfang = "<" + gesuchterTag + ">";
   string muster_ende   = "</" + gesuchterTag + ">";

   DateiHandleStruktur tempEinzulesendeDatei;
   tempEinzulesendeDatei.HandleEintragen( zuOeffnendeDatei, "r" );
                                                           // Datei zeilenweise lesen und nach dem gesuchten Tag überprüfen
   while ( !  tempEinzulesendeDatei.ZeileEinlesen( tempEinzulesendeDatei.HandleListeAusgeben()[0], zeile ) ) {
      size_t stelle_muster_anfang = zeile.find( muster_anfang );
      if ( stelle_muster_anfang != string::npos ) {        // Wenn ein Muster in der Zeile beginnt, untersuche die Zeile genauer
                                                           // und extrahiere jedes vollständige Muster
         string zeile_ab_muster = zeile.substr( stelle_muster_anfang );
         size_t stelle_muster_ende = zeile_ab_muster.find( muster_ende );
         if ( stelle_muster_ende == string::npos ) {       // Wurde das Dateiende erreicht, oder liegt ein (Formatierungs-)fehler vor?
            if ( feof( tempEinzulesendeDatei.HandleListeAusgeben()[0] ) ) {
               stelle_muster_ende = zeile_ab_muster.find( muster_ende.substr( 0, muster_ende.size()-1 ) );
               if ( stelle_muster_ende != string::npos ) { // Wenn das Dateiende erreicht wurde, teste einen um eins gekürzten String
                                                           // Entspricht er dem um eins gekürzten Endmuster, dann akzeptiere ihn
                  taginhalt = zeile_ab_muster.substr( muster_anfang.size(), stelle_muster_ende-muster_anfang.size() );
                  return 0;
               }
            }

            cout << "Abbruch: Tag " << muster_anfang << " nicht ordentlich definiert-fehlendes " + muster_ende + " in gleicher Zeile?\n";
            return 1;
         }

         taginhalt = zeile_ab_muster.substr( muster_anfang.size(), stelle_muster_ende-muster_anfang.size() );
         break;
      }
   }

   return 0;
}



// =================================================================
// ============   Z E I L E     U N T E R S U C H E N   ============
// =================================================================
// ---
int ZeileUntersuchenUndAusgeben( string &ausgangs_zeile, string &vorMuster, string &dasMuster, string &nachMuster, string muster_beginn, string muster_ende ) {
   vorMuster = "";
   dasMuster = "";
   nachMuster = "";

   size_t startzahl_muster = ausgangs_zeile.find( muster_beginn );

   if ( startzahl_muster != std::string::npos ) {          // Wenn ein Muster in der Zeile beginnt, untersuche die Zeile genauer
                                                           // und extrahiere jedes vollständige Muster
      string zeile_ab_muster = ausgangs_zeile.substr(startzahl_muster);
      size_t endzahl_muster = zeile_ab_muster.find( muster_ende );
      if ( endzahl_muster == std::string::npos ) {
         std::cout << "Abbruch: Variable " << zeile_ab_muster << " nicht ordentlich definiert-fehlendes \"" + muster_ende + "\" ?\n";
         return 1;
      }
                                                           // Ein Muster konnte erfolgreich extrahiert werden und wird in "muster" gespeichert
      dasMuster = zeile_ab_muster.substr( muster_beginn.size(), endzahl_muster-muster_beginn.size() );

                                                           // Gib alles vor dem Muster in die Datei aus und bearbeite anschließend das Muster
                                                           // in der Funktion "MusterErsetzen"
      vorMuster.append( ausgangs_zeile.substr( 0, startzahl_muster ) );

                                                           // Die restliche Zeile nach dem gerade bearbeiteten Muster könnte noch weitere Muster
                                                           // enthalten. Prüfe, ob noch Muster vorhanden sind und rufe die Funktion erneut auf
      nachMuster = zeile_ab_muster.substr( endzahl_muster+muster_ende.size() );
      return 0;
   }
   else
      vorMuster.append( ausgangs_zeile );                  // Enthält die Zeile kein Muster, dann schreibe die Zeile direkt in die Datei

   return 0;
}



// =================================================================
// ============   B E N U T Z U N G S H I N W E I S E   ============
// =================================================================
// ---
void zeige_benutzungshinweise() {
   cout << "Es werden benötigt:\n";
   cout << " - ein Ordner \"" << globalvar::ordner_einzulesen << "\", in dem sich mindestens eine Datei \"irgendwas.txt\" befindet\n";
   cout << " - ein Ordner \"" << globalvar::ordner_ausgabe << "\", in dem sich die Ordner \"" << globalvar::ordner_eintraege ;
   cout << "\" und \"" << globalvar::ordner_schlagwoerter << "\" befinden\n";
   cout << " - die Dateien \"dateianfang.txt\" und \"dateiende.txt\", die sich im gleichen Verzeichnis wie dieses Programm befinden müssen\n";
   cout << "---Ende--\n";
}



// =================================================================
// ==   E X I S T E N Z Ü B E R P R Ü F U N G   ( O R D N E R )   ==
// =================================================================
// ---
int ueberpruefe_existenz_von_ordner( string zu_ueberpruefender_ordner ) {
   struct stat sb;

   if (! ( stat( zu_ueberpruefender_ordner.c_str(), &sb ) == 0 && S_ISDIR( sb.st_mode ))) {
      cout << "Abbruch: Ordner " << zu_ueberpruefender_ordner << " nicht vorhanden\n";
      zeige_benutzungshinweise();
      return 1;
   }

   return 0;
}



// =================================================================
// =======   B E D I N G U N G E N     Ü B E R P R Ü F E N   =======
// =================================================================
// ---
int BedingungenUeberpruefen() {
   if ( ueberpruefe_existenz_von_ordner( globalvar::ordner_einzulesen ) )
      return 1;

   if ( ueberpruefe_existenz_von_ordner( globalvar::ordner_ausgabe ) )
      return 1;

   if ( ueberpruefe_existenz_von_ordner( globalvar::ordner_ausgabe + "/" + globalvar::ordner_eintraege ) )
      return 1;

   if ( ueberpruefe_existenz_von_ordner( globalvar::ordner_ausgabe + "/" + globalvar::ordner_schlagwoerter ) )
      return 1;
   return 0;
}



// =================================================================
// ===============   D A T E I E N     Ö F F N E N   ===============
// =================================================================
// ---
int DateiOeffnen( DateiHandleStruktur &meineDateiHandleStruktur, string dateiname, string zugriff ) {
   if (zugriff == "r" ) {                                  // Verwendung einer try-catch Struktur, um bei Dateizugriffsproblemen
      try {                                                // die Ressourcen (Klasse) wieder freizugeben und das Programm zu beenden
         meineDateiHandleStruktur.HandleEintragen( dateiname, "r" );
         return 0;
      }
      catch( ... ) {
         cout << "Abbruch: Datei \"" << dateiname << "\" konnte nicht zum Lesen geöffnet werden\n";
         return 1;
      }
   }
   else {
      try {
         meineDateiHandleStruktur.HandleEintragen( dateiname, "w" );
         return 0;
      }
      catch( ... ) {
         cout << "Abbruch: Datei \"" << dateiname << "\" konnte nicht zum Schreiben geöffnet werden\n";
         return 1;
      }
   }
}



// =================================================================
// ==========   D A T E I L I S T E     E I N L E S E N   ==========
// =================================================================
// ---
int LeseDateienAusVerzeichnis( list<string> &dateiliste ) {
   DIR *dir_pointer = NULL;                                // Erzeuge einen Pointer, der auf das Verzeichnis "quelldateien" zeigt
   dir_pointer = opendir( globalvar::ordner_einzulesen.c_str() );

   struct dirent *eintrag_pointer = NULL;

   if ( dir_pointer == NULL ) {
      cout << "Abbruch: dir_pointer konnte nicht richtig initialisiert werden\n";
      return 1;
   }

   while ( eintrag_pointer = readdir( dir_pointer ) ) {    // Für jede Datei, die gefunden wurde...
      if ( eintrag_pointer == NULL ) {
         cout << "Abbruch: eintrag_pointer konnte nicht richtig initialisiert werden";
         return 1;
      }

      string name = eintrag_pointer->d_name;               // Speichere den Dateinamen, auf dem der Pointer aktuell zeigt in "name"
      if ( name.size() <= 4 )                              // Dateinamen kleiner als vier Zeichen verwerfen (v.a. Linux Symlinks "." und "..")
         continue;

      dateiliste.push_back( name );                        // Dateinamen in Dateiliste speichern
   }

   closedir( dir_pointer );
}
// angelehnt an http://www.dreamincode.net/forums/topic/59943-accessing-directories-in-cc-part-i/



// =================================================================
// ====   I N T E R N E S     M U S T E R     E R S E T Z E N   ====
// =================================================================
// ---
void InternesMusterErsetzen( string &muster, vector<string> &pfadVon, string &rueckgabe ) {
   int musterCookie = 0;
   if ( muster == "zum_hauptverzeichnis" ) {
      rueckgabe.append( pfadVon[0] );
      musterCookie = 1;
   }

   if ( muster == "zu_den_schlagwoertern" ) {
      rueckgabe.append( pfadVon[1] );
      musterCookie = 1;
   }

   if ( muster == "zu_den_eintraegen" ) {
      rueckgabe.append( pfadVon[2] );
      musterCookie = 1;
   }

   if ( musterCookie == 0 )
      cout << "Warnung: Muster \"" << muster << "\" nicht implementiert" << endl;
}



// =================================================================
// =========   E R S T E L L E     Z I E L D A T E I E N   =========
// =================================================================
// ---
int ErstelleZieldateien( Gesamtstruktur &meineStruktur ) {
   int maxEintraegeIndex = _praep_maxEintraegeIndex_;      // Anzahl an Einträgen, die in die index Dateien geschrieben werden

   // --- Variablen mit den Namenslisten (zeitlich) und den Schlagworten
   vector<Eintrag> EintraegeZeitlich;                      // Diese Variable enthält gleich die zeitlich sortierte Liste
   vector<string>  Schlagwortliste;                        // Hier drin werden die alphabetisch sortierten Schlagwörter gespeichert
   vector<int>     AnzahlSchlagworte;                      // Enthält Häufigkeit der Schlagwörter. Wird beim Schreiben der Schlagwortdateien
                                                           // dekrementiert um zu erkennen, wann der letzte Eintrag pro Schlagwort erreicht ist
   EintraegeZeitlich = meineStruktur.EintraegeAusgebenDatum();
   Schlagwortliste   = meineStruktur.AusgabeSchlagworte();
   AnzahlSchlagworte = meineStruktur.AusgabeAnzahlSchlagworte();
   for ( int i = 0; i < AnzahlSchlagworte.size(); i++ )    // Da vier Varianten pro Datei geschrieben werden -> Anzahl anpassen
      AnzahlSchlagworte[i]=AnzahlSchlagworte[i]*4;
   // ----------------------------------------------------


   // --- Variablen für die relativen Pfade in den HTML Links
   vector<string> pfadVomHauptverzeichnis;
   vector<string> pfadVonSchlagwoertern;
   vector<string> pfadVonEintraegen;

   // (0) Zum Hauptverzeichnis
   pfadVomHauptverzeichnis.push_back( "" );
   pfadVonSchlagwoertern.push_back(   "../" );
   pfadVonEintraegen.push_back(       "../" );

   // (1) Zu den Schlagwörtern
   pfadVomHauptverzeichnis.push_back( globalvar::ordner_schlagwoerter + "/" );
   pfadVonSchlagwoertern.push_back(   "" );
   pfadVonEintraegen.push_back(       "../" + globalvar::ordner_schlagwoerter + "/" );

   // (2) Zu den Einträgen
   pfadVomHauptverzeichnis.push_back( globalvar::ordner_eintraege + "/" );
   pfadVonSchlagwoertern.push_back(   "../" + globalvar::ordner_eintraege + "/" );
   pfadVonEintraegen.push_back(       "" );
   // ----------------------------------------------------


   // --- Temporäre Variablen und DateiHandleStrukturen
   string dateisuffix[] = { "", "-al", "-ar", "-sc" };     // Es werden vier Dateivarianten bei Index- und Eintragsdateien
   string kernname;                                        // ( "name.html", "name-al.html", "name-ar.html", "name-sc.html" )
                                                           // und drei bei Schlagwortdateien (keine "name-sc.html") verwendet

   DateiHandleStruktur IndexdateienHandles;                // Dateihandles aller Indexdateien
   DateiHandleStruktur SchlagwortdateienHandles;           // Dateihandles aller Schlagwortdateien
   DateiHandleStruktur EintragdateienHandles;              // Dateihandles aller Eintragsdateien

   DateiHandleStruktur VorlagedateienHandles;              // Dateihandles der Vorlagedateien gespeichert
   DateiHandleStruktur QuelldateienHandles;                // Dateihandles der Quelldateien gespeichert

   if ( DateiOeffnen( VorlagedateienHandles, "dateianfang.txt", "r" ) )
      return 1;
   if ( DateiOeffnen( VorlagedateienHandles, "dateiende.txt", "r" ) )
      return 1;

   string tempDateiname;                                   // temporäre Pfadangaben und Dateinamen
   string tempZeile;                                       // temporäre Variable für eine oder mehrere Zeilen (Ausgabe-)text
   string tempDateiInhalt = "";                            // temporäre Variable für vollständig eingelesene Dateien

   // Hilfsvariablen zur Ersetzung von Mustern im eingelesenen Text
   string vorMuster  = "";
   string dasMuster  = "";

   DateiHandleStruktur tempDateiHandle;

   string nachMuster = "";

   FILE* AktuellZuLesendeDateiHandle;
   FILE* AktuellZuSchreibendeDateiHandle;

   cout << "Schreibe Dateien...\n";
   // ----------------------------------------------------


   // --- Indexdateien zum Schreiben öffnen und Dateianfang hinein schreiben
   for ( int variantnr = 0; variantnr < 4; variantnr++ ) { // Sobald eine Datei nicht gelesen werden kann -> Abbruch
      kernname = "index";
      tempDateiname = globalvar::ordner_ausgabe + "/" + kernname + dateisuffix[variantnr] + ".html";
                                                           // Handle über Aufruf von DateiOeffnen in IndexdateienHandles speichern
      if ( DateiOeffnen( IndexdateienHandles, tempDateiname, "w" ) )
         return 1;

      AktuellZuLesendeDateiHandle     = VorlagedateienHandles.HandleListeAusgeben()[0];
      AktuellZuSchreibendeDateiHandle = IndexdateienHandles.HandleListeAusgeben()[variantnr];

      tempDateiInhalt = VorlagedateienHandles.DateiEinlesen( AktuellZuLesendeDateiHandle, meineStruktur, kernname, dateisuffix[variantnr], pfadVomHauptverzeichnis, 0 );
      IndexdateienHandles.ZeichenAusgeben( AktuellZuSchreibendeDateiHandle, tempDateiInhalt );
   }
   // ----------------------------------------------------


   // --- Schlagwortdateien zum Schreiben öffnen und Dateianfang hinein schreiben
   for ( int schlagwortnr = 0; schlagwortnr < Schlagwortliste.size(); schlagwortnr++ ) {
      kernname = Schlagwortliste[schlagwortnr];
      transform( kernname.begin(), kernname.end(), kernname.begin(), ::tolower );

      for ( int variantnr = 0; variantnr < 4; variantnr++ ) {
         tempDateiname = globalvar::ordner_ausgabe + "/" + globalvar::ordner_schlagwoerter + "/schlagwort-" + kernname + dateisuffix[variantnr] + ".html";

         if ( DateiOeffnen( SchlagwortdateienHandles, tempDateiname, "w" ) )
            return 1;

         AktuellZuLesendeDateiHandle     = VorlagedateienHandles.HandleListeAusgeben()[0];
         AktuellZuSchreibendeDateiHandle = SchlagwortdateienHandles.HandleListeAusgeben()[schlagwortnr*4+variantnr];

         tempDateiInhalt = VorlagedateienHandles.DateiEinlesen( AktuellZuLesendeDateiHandle, meineStruktur, kernname, dateisuffix[variantnr], pfadVonSchlagwoertern, 1 );
         SchlagwortdateienHandles.ZeichenAusgeben( AktuellZuSchreibendeDateiHandle, tempDateiInhalt );
      }
   }
   // ----------------------------------------------------


   // Eintragsdateien zum Schreiben öffnen, Dateianfang hinein schreiben, Inhalt der Quelldateien auslesen
   // ggf. Muster ersetzen und in ALLE offenen Dateien schreiben, Dateiende in Eintragsdateien schreiben und schließen
   for ( int eintragnr = 0; eintragnr < EintraegeZeitlich.size(); eintragnr++ ) {
      kernname = EintraegeZeitlich[eintragnr].HoleKernname();
      tempDateiname = globalvar::ordner_einzulesen + "/" + kernname + ".txt";

      // Die einzelnen Schlagwörter (nochmal) extrahieren, die zu jedem Eintrag gehören
      vector<string> schlagwortVec;
      vector<string> schlagwortVecGross;
      string temp_schlagworte = EintraegeZeitlich[eintragnr].HoleSchlagwoerter();
      string temp_schlagworte_klein = temp_schlagworte;
      size_t stelle_komma = temp_schlagworte.find( ", " );

      if ( stelle_komma != std::string::npos ) {
         while ( stelle_komma != std::string::npos ) {
            string temp_anfang = temp_schlagworte.substr( 0, stelle_komma );
                                                           // Starte zwei Stellen weiter wegen dem Trennzeichen ", "
            temp_schlagworte_klein = temp_anfang;
            schlagwortVecGross.push_back ( temp_anfang );
            transform( temp_schlagworte_klein.begin(), temp_schlagworte_klein.end(), temp_schlagworte_klein.begin(), ::tolower );
            schlagwortVec.push_back( temp_schlagworte_klein );
            temp_schlagworte = temp_schlagworte.substr( stelle_komma+2 );
            stelle_komma = temp_schlagworte.find( ", " );
         }
      }

      temp_schlagworte_klein = temp_schlagworte;
      schlagwortVecGross.push_back ( temp_schlagworte );
      transform( temp_schlagworte_klein.begin(), temp_schlagworte_klein.end(), temp_schlagworte_klein.begin(), ::tolower );
      schlagwortVec.push_back( temp_schlagworte_klein );
      // ----------------------------------------------------

      if ( DateiOeffnen( QuelldateienHandles, tempDateiname , "r" ) )
         return 1;

      for ( int variantnr = 0; variantnr < 4; variantnr++ ) {
         tempDateiname = globalvar::ordner_ausgabe + "/" + globalvar::ordner_eintraege + "/" + kernname + dateisuffix[variantnr] + ".html";

         if ( DateiOeffnen( EintragdateienHandles, tempDateiname, "w" ) )
            return 1;

         // Zuerst Einlesen der Vorlagedateien
         AktuellZuLesendeDateiHandle     = VorlagedateienHandles.HandleListeAusgeben()[0];
         AktuellZuSchreibendeDateiHandle = EintragdateienHandles.HandleListeAusgeben()[eintragnr*4+variantnr];

         tempDateiInhalt = VorlagedateienHandles.DateiEinlesen( AktuellZuLesendeDateiHandle, meineStruktur, kernname, dateisuffix[variantnr], pfadVonEintraegen, 0 );
         EintragdateienHandles.ZeichenAusgeben( AktuellZuSchreibendeDateiHandle, tempDateiInhalt );

         AktuellZuLesendeDateiHandle     = QuelldateienHandles.HandleListeAusgeben()[eintragnr];
         // ----------------------------------------------------


         // ---------------------------------------------------- Beginn des Eintrags
         tempDateiInhalt = "";                             // Den ganzen fertigen Eintrag in dieser Variablen speichern
         // Einschub: Überschrift und Datum

         tempDateiInhalt.append( "\n         <div class=\"eintrag\">\n" );
         tempDateiInhalt.append( "            <div class=\"ueberschrift\">\n" );
         tempDateiInhalt.append( "               <div class=\"h1\"><a href=\"=__zu_den_eintraegen__=" + kernname + ".html\">" );
         tempDateiInhalt.append( EintraegeZeitlich[eintragnr].HoleTitel()+ "</a></div>\n" );
         tempDateiInhalt.append( "               <div class=\"datum\">" );
         tempDateiInhalt.append( EintraegeZeitlich[eintragnr].HoleDatum() + "</div>\n" );
         tempDateiInhalt.append( "            </div>\n" );
         tempDateiInhalt.append( "            <div class=\"text\">\n" );

         // Jetzt Einlesen der Quelldateien
         int imInhaltsbereich = 0;
         string inhalt_beginn = "<inhalt>";
         string inhalt_ende   = "</inhalt>";
         size_t stelle_inhalt_beginn;
         size_t stelle_inhalt_ende;

         // Lokale Variablen für den Inhaltsbereich
         string tempMuster      = "";
         string tempVorMuster   = "";
         string tempNachMuster  = "";

         string tempZaehler = "";
         string temp_muster_beginn = "<fussnote>";
         string temp_muster_ende   = "</fussnote>";

         string zuSchreiben;
         vector<string> textFussnoten;                     // Inhalt der aus dem Text extrahierten Fussnoten; da in der Schleife
         int fussnotebeenden = 0;                          // erstellt, müssen die Variablen am Ende nicht zu null gesetzt werden

         while ( ! QuelldateienHandles.ZeileEinlesen( AktuellZuLesendeDateiHandle, tempZeile ) ) {
            stelle_inhalt_beginn = tempZeile.find( inhalt_beginn );
            stelle_inhalt_ende = tempZeile.find( inhalt_ende );

            if ( imInhaltsbereich == 2 )                   // Nachdem der <inhalt> Tag eingelesen wird,
               imInhaltsbereich = 1;                       // soll erst die NÄCHSTE Zeile ausgegeben werden
                                                           // Deshalb wird imInhaltsbereich ...
            if ( stelle_inhalt_beginn != std::string::npos )
               imInhaltsbereich = 2;                       // erst auf 2 und weiter oben auf 1 gesetzt

            if ( stelle_inhalt_ende != std::string::npos )
               imInhaltsbereich = 0;

            // Fußnotenmanagement
            if ( imInhaltsbereich == 1 ) {
               while ( tempZeile.size() != 0 ) {
                  ZeileUntersuchenUndAusgeben( tempZeile, tempVorMuster, tempMuster, tempNachMuster, temp_muster_beginn, temp_muster_ende );

                  if ( tempMuster != "" ) {
                     textFussnoten.push_back( tempMuster );
                     tempZaehler = IntegerToString( textFussnoten.size() );

                     if ( ( ( tempVorMuster == "" ) || ( tempVorMuster == "," ) ) && ( textFussnoten.size() > 1 ) ) {
                        zuSchreiben = ",<a href=\"#fn-" + kernname + tempZaehler + "\" id=\"ref-" + kernname + tempZaehler + "\">" + tempZaehler + "</a>";
                        tempDateiInhalt.append( zuSchreiben );
                        fussnotebeenden = 1;
                     }
                     else {
                        if ( fussnotebeenden == 1 ) {
                           tempDateiInhalt.append( "</sup>" );
                           fussnotebeenden = 0;
                        }
                        tempDateiInhalt.append( tempVorMuster );
                        zuSchreiben = "<sup><a href=\"#fn-" + kernname + tempZaehler + "\" id=\"ref-" + kernname + tempZaehler + "\">" + tempZaehler + "</a>";
                        tempDateiInhalt.append( zuSchreiben );
                        fussnotebeenden = 1;
                     }
                  }
                  else {
                     if ( fussnotebeenden == 1 ) {
                        tempDateiInhalt.append( "</sup>" );
                        fussnotebeenden = 0;
                     }
                     tempDateiInhalt.append( tempVorMuster );
                  }

                  tempZeile = tempNachMuster;
               }

               if ( fussnotebeenden == 1 )
                  tempDateiInhalt.append( "</sup>" );

               tempDateiInhalt.append( "\n" );
            }
         }

         // Einschub: Anhang - Schlagwörter ausgeben
         tempZeile = "               <br />\n            </div>\n            <div class=\"anhang\">\n               <ul class=\"schlagwoerter\">\n";
         tempZeile.append ( "                  <li class=\"first\"><a href=\"#top\" title=\"Zum Seitenanfang\">&uarr;</a></li>\n" );
         tempZeile.append ( "                  <li class=\"first\">Schlagwörter:</li>\n" );
         tempDateiInhalt.append( tempZeile );

         for ( int zaehler_schlagworte = 0; zaehler_schlagworte < schlagwortVec.size(); zaehler_schlagworte++ ) {
            tempZeile = "                  <li><a href=\"=__zu_den_schlagwoertern__=schlagwort-" + schlagwortVec[zaehler_schlagworte] + ".html\">";
            tempZeile.append ( schlagwortVecGross[zaehler_schlagworte] + "</a></li>\n" );
            tempDateiInhalt.append( tempZeile );
         }
         tempDateiInhalt.append( "               </ul>\n" );


         // Einschub: Anhang - Fussnoten, wenn vorhanden
         if ( textFussnoten.size() != 0 ) {
            int zaehlerfussnoten = 1;
            string tempHTMLausdruck;
            tempDateiInhalt.append( "               <hr />\n               <ol class=\"fussnoten\">\n" );

            while ( zaehlerfussnoten <= textFussnoten.size() ) {
               tempHTMLausdruck = "                  <li><span id=\"fn-" + kernname + IntegerToString( zaehlerfussnoten ) + "\">";
               tempDateiInhalt.append( tempHTMLausdruck );
               tempHTMLausdruck = textFussnoten[zaehlerfussnoten-1] + " -- <a href=\"#ref-" + kernname + IntegerToString( zaehlerfussnoten );
               tempDateiInhalt.append( tempHTMLausdruck );
               tempDateiInhalt.append( "\" title=\"Zurück zur Textstelle\">&uarr;</a></span></li>\n" );
               zaehlerfussnoten++;
            }
            tempDateiInhalt.append( "               </ol>\n" );
         }

         tempDateiInhalt.append( "            </div>\n         </div>\n\n" );
         // ---------------------------------------------------- Ende des Eintrags


         // --- Ausgeben des Eintrags in alle relevanten Dateien
         string eintragDateiInhalt = tempDateiInhalt;      // Der gesamte Eintrag steht in tempDateiInhalt, muss aber noch je nach
         string indexDateiInhalt = tempDateiInhalt;        // Zieldatei angepasst werden (enthält interne Muster wie =__zu_den_eintraegen__=

         string ausgabe = "";                              // Es wird ein Ausgabestring definiert, der bisherige Inhalt nach Mustern untersucht
         while ( eintragDateiInhalt.size() != 0 ) {        // und die fertige Ausgabe mit ZeichenAusgeben( ..., ausgabe ) in die Datei geschrieben
            ZeileUntersuchenUndAusgeben( eintragDateiInhalt, tempVorMuster, tempMuster, tempNachMuster, "=__", "__=" );

            ausgabe.append( tempVorMuster );
            if ( tempMuster != "" )
               InternesMusterErsetzen( tempMuster, pfadVonEintraegen, ausgabe );

            eintragDateiInhalt = tempNachMuster;
         }
         EintragdateienHandles.ZeichenAusgeben( AktuellZuSchreibendeDateiHandle, ausgabe );

         if ( eintragnr < maxEintraegeIndex ) {            // Jetzt analog das Schreiben der Indexdateien
            ausgabe = "";
            while ( indexDateiInhalt.size() != 0 ) {
               ZeileUntersuchenUndAusgeben( indexDateiInhalt, tempVorMuster, tempMuster, tempNachMuster, "=__", "__=" );

               ausgabe.append( tempVorMuster );
               if ( tempMuster != "" )
                  InternesMusterErsetzen( tempMuster, pfadVomHauptverzeichnis, ausgabe );

               indexDateiInhalt = tempNachMuster;
            }
            if ( ( eintragnr < maxEintraegeIndex-1 ) && ( eintragnr < EintraegeZeitlich.size()-1 ) )
               IndexdateienHandles.ZeichenAusgeben( IndexdateienHandles.HandleListeAusgeben()[variantnr], ausgabe + "         <br />\n         <br />\n" );
            else
               IndexdateienHandles.ZeichenAusgeben( IndexdateienHandles.HandleListeAusgeben()[variantnr], ausgabe );
         }

         size_t startzahl_schlagwort;                      // Bei den Schlagwortdateien muss erstmal zu den passenden Schlagwort die entsprechende
         for ( int i = 0; i < schlagwortVec.size(); i++ ) {// geöffnete Datei gefunden werden. Dann Vorgehen wie bei Einträgen- und Indexdateien
            for ( int k = 0; k < Schlagwortliste.size(); k++ ) {
               string schlagwort_klein = Schlagwortliste[k];
               transform( schlagwort_klein.begin(), schlagwort_klein.end(), schlagwort_klein.begin(), ::tolower );
               startzahl_schlagwort = schlagwort_klein.find( schlagwortVec[i] );

                                                           // Das Schlagwort muss genau dem Eintrag aus dem Schlagwortvektor entsprechen (Übereinstimmung+gleiche Länge)
               if ( startzahl_schlagwort != std::string::npos ) {
                  if ( schlagwort_klein.size() == schlagwortVec[i].size() ) {
                     string schlagwortDateiInhalt = tempDateiInhalt;

                     ausgabe = "";
                     while ( schlagwortDateiInhalt.size() != 0 ) {
                        ZeileUntersuchenUndAusgeben( schlagwortDateiInhalt, tempVorMuster, tempMuster, tempNachMuster, "=__", "__=" );

                        ausgabe.append( tempVorMuster );
                        if ( tempMuster != "" )
                           InternesMusterErsetzen( tempMuster, pfadVonSchlagwoertern, ausgabe );

                        schlagwortDateiInhalt = tempNachMuster;
                     }                                        // Hinter jedem Eintrag in einer Schlagwortdatei zwei Leerzeilen Abstand anhängen,
                     if ( AnzahlSchlagworte[k] > 4 ) {        // sofern es nicht der letzte Eintrag ist
                        SchlagwortdateienHandles.ZeichenAusgeben( SchlagwortdateienHandles.HandleListeAusgeben()[k*4+variantnr], ausgabe + "         <br />\n         <br />\n" );
                        AnzahlSchlagworte[k]--;
                     }
                     else
                        SchlagwortdateienHandles.ZeichenAusgeben( SchlagwortdateienHandles.HandleListeAusgeben()[k*4+variantnr], ausgabe );
                  }
               }
            }
         }
         // ----------------------------------------------------


         // --- Und zum Schluss die Vorlagen für das Dateiende der Eintragsdateien
         AktuellZuLesendeDateiHandle     = VorlagedateienHandles.HandleListeAusgeben()[1];

         tempDateiInhalt = VorlagedateienHandles.DateiEinlesen( AktuellZuLesendeDateiHandle, meineStruktur, kernname, dateisuffix[variantnr], pfadVonEintraegen, 0 );
         EintragdateienHandles.ZeichenAusgeben( AktuellZuSchreibendeDateiHandle, tempDateiInhalt );
         // ----------------------------------------------------
      }
   }


   // --- Dateiende in Schlagwortdateien schreiben
   for ( int schlagwortnr = 0; schlagwortnr < Schlagwortliste.size(); schlagwortnr++ ) {
      kernname = Schlagwortliste[schlagwortnr];

      for ( int variantnr = 0; variantnr < 4; variantnr++ ) {
         AktuellZuLesendeDateiHandle     = VorlagedateienHandles.HandleListeAusgeben()[1];
         AktuellZuSchreibendeDateiHandle = SchlagwortdateienHandles.HandleListeAusgeben()[schlagwortnr*4+variantnr];

         tempDateiInhalt = VorlagedateienHandles.DateiEinlesen( AktuellZuLesendeDateiHandle, meineStruktur, kernname, dateisuffix[variantnr], pfadVonSchlagwoertern, 1 );
         SchlagwortdateienHandles.ZeichenAusgeben( AktuellZuSchreibendeDateiHandle, tempDateiInhalt );
      }
   }
   // ----------------------------------------------------


   // --- Dateiende in Indexdateien schreiben
   for ( int variantnr = 0; variantnr < 4; variantnr++ ) {
      kernname = "index";
      AktuellZuLesendeDateiHandle     = VorlagedateienHandles.HandleListeAusgeben()[1];
      AktuellZuSchreibendeDateiHandle = IndexdateienHandles.HandleListeAusgeben()[variantnr];

      tempDateiInhalt = VorlagedateienHandles.DateiEinlesen( AktuellZuLesendeDateiHandle, meineStruktur, kernname, dateisuffix[variantnr], pfadVomHauptverzeichnis, 0 );
      IndexdateienHandles.ZeichenAusgeben( AktuellZuSchreibendeDateiHandle, tempDateiInhalt );
   }
   // ----------------------------------------------------

   cout << "Fertig\n";
   return 0;
}



// =================================================================
// ==================   H A U P T R O U T I N E   ==================
// =================================================================
// ---
int main( int argc, char* argv[] ) {
   if ( BedingungenUeberpruefen() )                        // Voraussetzungen am Dateisystem für die Bearbeitung überprüfen
      return 1;

   // --- Erstelle eine alphabetische und zeitliche Struktur mit den Namen und Taginhalt sämlticher zu bearbeitenden Dateien
   list<string> dateiliste;                                // Erzeuge eine Dateiliste mit den Namen aller Dateien
   if ( LeseDateienAusVerzeichnis( dateiliste ) )          // aus dem Quellordner
      return 1;
   dateiliste.sort();                                      // Sortiere die Daten alphabetisch (Annahme: Alle klein geschrieben)

   Gesamtstruktur meineStruktur;                           // Speichere die Dateinamen und Informationen
   list<string>::iterator zaehler_dateiliste;              // aus den Dateien in Einträgen von meineStruktur
   for ( zaehler_dateiliste = dateiliste.begin(); zaehler_dateiliste != dateiliste.end(); zaehler_dateiliste++ ) {
      if ( meineStruktur.EintragHinzufuegen( *zaehler_dateiliste ) )
         return 1;                                         // Falls bei der Zuordnung etwas schief geht, breche ab
   }

   meineStruktur.ErstelleInterneAlphabetischeliste();      // Zuerst soll eine alphabetische Liste erstellt werden
   meineStruktur.ErstelleInterneDatumsliste();             // Anschließend in der Klasse eine zeitlich sortierte Liste erstellen
   meineStruktur.ErstelleInterneSchlagwortliste();         // und eine sortierte Schlagwortliste
   // ----------------------------------------------------


   // --- Verarbeite alle in der Klasse erzeugten Listen für die Ausgabe in ErstelleZieldateien
   if ( ErstelleZieldateien( meineStruktur ) )             // Der Übersichtlichkeit halber wird der Rest in der Funktion
      return 1;                                            // ErstelleZieldateien erledigt, könnte aber sonst auch hier stehen.

   cout << "---Ende--\n";
   return 0;
}
